@@ -321,7 +321,7 @@ pub fn einsum_nest_optimize(einsum: &Einsum) -> Einsum {
 
 #[pyfunction]
 pub fn deep_optimize_einsums(circ: CircuitRc) -> CircuitRc {
-    deep_map_unwrap(&circ, &|x| x.map_or_clone(einsum_nest_optimize))
+    deep_map_unwrap(&circ, |x| x.map_or_clone(einsum_nest_optimize))
 }
 
 /// corresponds to python function `construct_diag`
