@@ -1,5 +1,6 @@
 use super::{
     deep_map, visit_circuit_fallable, Circuit, CircuitConstructionError, CircuitNode, CircuitRc,
+    HashBytes,
 };
 use crate::circuit::deep_map_preorder;
 use crate::circuit::deep_map_unwrap_preorder;
@@ -36,13 +37,23 @@ pub fn filter_nodes_py(
 }
 
 #[pyfunction]
-pub fn replace_nodes(circuit: CircuitRc, map: HashMap<CircuitRc, CircuitRc>) -> CircuitRc {
+#[pyo3(name = "replace_nodes")]
+pub fn replace_nodes_py(circuit: CircuitRc, map: HashMap<CircuitRc, CircuitRc>) -> CircuitRc {
     deep_map_unwrap_preorder(&**circuit, |x: &Circuit| -> CircuitRc {
         let rc = x.clone().rc();
         map.get(&rc).cloned().unwrap_or(rc)
     })
 }
 
+/// Replaces child nodes recursively with a mapping
+/// In pre-order so that high level nodes are replaced first
+pub fn replace_nodes(circuit: CircuitRc, map: &HashMap<HashBytes, CircuitRc>) -> CircuitRc {
+    deep_map_unwrap_preorder(&**circuit, |x: &Circuit| -> CircuitRc {
+        let rc = x.clone().rc();
+        map.get(&rc.info().hash).cloned().unwrap_or(rc)
+    })
+}
+
 #[pyfunction]
 #[pyo3(name = "deep_map_preorder")]
 pub fn deep_map_preorder_py(
