@@ -1,3 +1,7 @@
+use std::collections::HashMap;
+
+use num_bigint::BigUint;
+
 use crate::{circuit::print::rust_expression_notation_circuit, pyo3_prelude::*};
 
 use super::{
@@ -5,6 +9,7 @@ use super::{
     canonicalize::deep_canonicalize,
     compiler_heuristics::deep_maybe_distribute,
     compiler_strip::strip_names,
+    debugging::{create_circuit_test, randn_ablate_great_grandchildren},
     deep_rewrite::deep_heuristic_nest_adds,
     deep_rewrite::{compiler_simp, deep_push_down_index},
     flat_concat,
@@ -12,7 +17,7 @@ use super::{
     scheduled_execution::{
         circuit_to_schedule, circuit_to_schedule_naive_toposort, evaluate_schedule, SchedulingError,
     },
-    CircuitNode, CircuitRc,
+    Circuit, CircuitNode, CircuitRc, HashBytes,
 };
 use crate::{
     py_types::{Tensor, UN_FLAT_CONCAT},
@@ -48,6 +53,8 @@ pub struct OptimizationSettings {
     pub numerical_scale_max: f64,
     #[pyo3(get, set)]
     pub capture_and_print: bool,
+    #[pyo3(get, set)]
+    pub create_tests: bool,
 }
 
 impl Default for OptimizationSettings {
@@ -63,6 +70,7 @@ impl Default for OptimizationSettings {
             numerical_scale_min: 1e-8,
             numerical_scale_max: 1e8,
             capture_and_print: false,
+            create_tests: false,
         }
     }
 }
@@ -80,7 +88,8 @@ impl OptimizationSettings {
         adjust_numerical_scale = "OptimizationSettings::default().adjust_numerical_scale",
         numerical_scale_min = "OptimizationSettings::default().numerical_scale_min",
         numerical_scale_max = "OptimizationSettings::default().numerical_scale_max",
-        capture_and_print = "OptimizationSettings::default().capture_and_print"
+        capture_and_print = "OptimizationSettings::default().capture_and_print",
+        create_tests = "OptimizationSettings::default().create_tests"
     )]
     fn new(
         verbose: usize,
@@ -93,6 +102,7 @@ impl OptimizationSettings {
         numerical_scale_min: f64,
         numerical_scale_max: f64,
         capture_and_print: bool,
+        create_tests: bool,
     ) -> Self {
         Self {
             verbose,
@@ -105,33 +115,84 @@ impl OptimizationSettings {
             numerical_scale_min,
             numerical_scale_max,
             capture_and_print,
+            create_tests,
+        }
+    }
+}
+
+#[derive(Debug, Clone)]
+pub struct OptimizationCache {
+    pub simplified: HashMap<HashBytes, CircuitRc>,
+    pub distributed: HashMap<HashBytes, Option<CircuitRc>>,
+    pub flops: HashMap<HashBytes, BigUint>,
+    pub sum_of_node_sizes: HashMap<HashBytes, BigUint>,
+}
+
+impl Default for OptimizationCache {
+    fn default() -> Self {
+        Self {
+            simplified: HashMap::new(),
+            distributed: HashMap::new(),
+            flops: HashMap::new(),
+            sum_of_node_sizes: HashMap::new(),
+        }
+    }
+}
+#[derive(Debug, Clone, Default)]
+pub struct OptimizationContext {
+    pub cache: OptimizationCache,
+    pub settings: OptimizationSettings,
+}
+impl OptimizationContext {
+    pub fn new_settings(settings: OptimizationSettings) -> Self {
+        Self {
+            cache: Default::default(),
+            settings,
         }
     }
 }
 
 #[pyfunction]
-pub fn optimize_circuit(circuit: CircuitRc, settings: OptimizationSettings) -> CircuitRc {
-    let print_timings = settings.verbose >= 2;
+#[pyo3(name = "optimize_circuit")]
+pub fn optimize_circuit_py(circuit: CircuitRc, settings: OptimizationSettings) -> CircuitRc {
+    let mut context: OptimizationContext = Default::default();
+    context.settings = settings;
+    optimize_circuit(circuit, &mut context)
+}
+
+pub fn optimize_circuit(circuit: CircuitRc, context: &mut OptimizationContext) -> CircuitRc {
+    let print_timings = context.settings.verbose >= 2;
     let circuit = timed!(strip_names(circuit), 10, print_timings);
     let circuit = timed!(deep_canonicalize(circuit), 10, print_timings);
-    if settings.verbose >= 4 {
+    if context.settings.verbose >= 4 {
         println!("Original Circuit");
         circuit.compiler_print();
     }
-    let circuit = timed!(compiler_simp(&circuit), 10, print_timings);
+    if context.settings.create_tests {
+        // Make compiler_simp tests here
+        let ablated = randn_ablate_great_grandchildren(circuit.clone());
+        create_circuit_test(
+            "compiler_simp",
+            ablated.clone(),
+            compiler_simp(&ablated, context),
+        )
+        .unwrap();
+    }
+    let circuit = timed!(compiler_simp(&circuit, context), 10, print_timings);
     // originally tried push_down_index in compiler_simp, but that produced worse circuits
     // for unknown reasons, maybe i'll investigate
     let circuit = timed!(deep_push_down_index(circuit, None), 10, print_timings);
-    let circuit = timed!(compiler_simp(&circuit), 10, print_timings);
+    let circuit = timed!(compiler_simp(&circuit, context), 10, print_timings);
     let circuit = timed!(deep_canonicalize(circuit), 10, print_timings);
-    if settings.verbose >= 3 {
+    if context.settings.verbose >= 3 {
         println!("Simplified Circuit");
         circuit.compiler_print();
     }
     let circuit = timed!(
-        apply_fn_until_same(&circuit, |x| compiler_simp(&deep_maybe_distribute(
-            x, settings
-        ))),
+        apply_fn_until_same(&circuit, |x| {
+            let distributed = deep_maybe_distribute(x, context);
+            compiler_simp(&distributed, context)
+        }),
         10,
         print_timings
     );
@@ -142,7 +203,7 @@ pub fn optimize_circuit(circuit: CircuitRc, settings: OptimizationSettings) -> C
     let circuit = timed!(deep_optimize_einsums(circuit), 10, print_timings);
 
     let circuit = timed!(deep_canonicalize(circuit), 10, print_timings);
-    if settings.verbose >= 3 {
+    if context.settings.verbose >= 3 {
         println!("Final Circuit");
         circuit.compiler_print();
     }
@@ -154,6 +215,10 @@ pub fn optimize_and_evaluate(
     circuit: CircuitRc,
     settings: OptimizationSettings,
 ) -> Result<Tensor, SchedulingError> {
+    let mut context = OptimizationContext {
+        settings,
+        cache: Default::default(),
+    };
     let verbose = settings.verbose;
     if verbose > 0 {
         println!("Optimizing verbose {}", verbose)
@@ -161,7 +226,7 @@ pub fn optimize_and_evaluate(
     if settings.capture_and_print {
         println!("{}", rust_expression_notation_circuit(circuit.clone()));
     }
-    let optimized_circuit = timed!(optimize_circuit(circuit, settings), 10, verbose >= 1);
+    let optimized_circuit = timed!(optimize_circuit(circuit, &mut context), 10, verbose >= 1);
     // return evaluate_fn(&*optimized_circuit).unwrap();
     let schedule = if settings.scheduling_naive {
         circuit_to_schedule_naive_toposort(optimized_circuit.clone())
