@@ -11,8 +11,8 @@ use crate::{
 use num_bigint::BigUint;
 
 use super::{
-    deep_map_unwrap, visit_circuit, ArrayConstant, Circuit, CircuitNode, CircuitRc, HashBytes,
-    Index,
+    apply_fn_cache, deep_map_unwrap, visit_circuit, ArrayConstant, Circuit, CircuitNode, CircuitRc,
+    HashBytes, Index,
 };
 use crate::tensor_util::TorchDeviceDtype;
 
@@ -69,6 +69,15 @@ pub fn total_flops(circuit: CircuitRc) -> BigUint {
     result
 }
 
+pub fn total_flops_cached(circuit: CircuitRc, cache: &mut HashMap<HashBytes, BigUint>) -> BigUint {
+    apply_fn_cache(
+        &circuit,
+        |x| total_flops(x.clone()),
+        cache,
+        |x| x.info().hash,
+    )
+}
+
 #[pyfunction]
 pub fn total_arrayconstant_size(circuit: CircuitRc) -> BigUint {
     let mut result: BigUint = BigUint::from(0usize);
@@ -79,6 +88,7 @@ pub fn total_arrayconstant_size(circuit: CircuitRc) -> BigUint {
     });
     result
 }
+
 #[pyfunction]
 pub fn sum_of_node_sizes(circuit: CircuitRc) -> BigUint {
     let mut result: BigUint = BigUint::from(0usize);
@@ -90,6 +100,18 @@ pub fn sum_of_node_sizes(circuit: CircuitRc) -> BigUint {
     result
 }
 
+pub fn sum_of_node_sizes_cached(
+    circuit: CircuitRc,
+    cache: &mut HashMap<HashBytes, BigUint>,
+) -> BigUint {
+    apply_fn_cache(
+        &circuit,
+        |x| sum_of_node_sizes(x.clone()),
+        cache,
+        |x| x.info().hash,
+    )
+}
+
 #[pyfunction]
 pub fn get_leaves(circuit: CircuitRc) -> Vec<CircuitRc> {
     let mut result: Vec<CircuitRc> = vec![];
