@@ -5,7 +5,6 @@ use itertools::Itertools;
 use num_bigint::BigUint;
 
 use crate::{
-    cached_circuit_lambda,
     circuit::diag_rewrite::{add_pull_diags, einsum_push_down_trace},
     util::{apply_fn_until_same, mapping_until_end, AsOp},
 };
@@ -13,6 +12,7 @@ use crate::{
 use super::{
     algebraic_rewrite::*,
     canonicalize::{canonicalize_node, deep_canonicalize},
+    circuit_optimizer::{OptimizationCache, OptimizationContext},
     concat_rewrite::add_pull_concat,
     concat_rewrite::{
         concat_drop_size_zero, concat_fuse, einsum_pull_concat, generalfunction_pull_concat,
@@ -137,84 +137,64 @@ pub fn compiler_simp_step(circuit: &Circuit) -> Option<CircuitRc> {
     }
 }
 
-// deep simplification strategy
-
-// if you just do preorder or postorder mapping, you might need to run simp O(circuit_depth) times
-// because pull_removable_axes or such could bubble up
-// this would make simp to convergence O(circuit_nodes*circuit_depth)
-
-// what's currently implemented is: simplify from bottom up, whenever a node is changed by simplification,
-// simplify its children and grandchildren, going down farther if those change
-// currently all simplification rewrites only change the grandchildren, not great-grandchildren
-// so this converges on one pass, and takes O(circuit_nodes+simplification_steps_needed) time
-
-// a better algorithm would be bottom up, whenever one changes check down until you see nodes that
-// you've simplified before. that would remove the dont-change-great-grandchildren requirement
-
-pub fn compiler_simp_down_if_here(circ: &Circuit) -> Option<CircuitRc> {
-    compiler_simp_step(circ).and_then(|x| compiler_simp_down(&x))
-}
-
-pub fn compiler_simp_down_one(circ: &Circuit) -> Option<CircuitRc> {
-    let simp_here = compiler_simp_step(circ);
-    if let Some(x) = simp_here {
-        return Some(compiler_simp_down(&x).unwrap_or(x));
-    }
-    circ.map_children_op(&mut compiler_simp_down_if_here)
-        .map(|x| x.rc())
-}
-
-pub fn compiler_simp_down(circ: &Circuit) -> Option<CircuitRc> {
-    // if remaining_stack().unwrap()<3_000_000{
-    //     println!("jank recursion limit reached");
-    //     circ.compiler_print();
-    //     panic!();
-    // }
-    let simp_here = compiler_simp_step(circ);
-    if let Some(x) = simp_here {
-        return Some(compiler_simp_down(&x).unwrap_or(x));
-    }
-    circ.map_children_op(&mut compiler_simp_down_one)
-        .map(|x| x.rc())
-}
-
+/// Deep simplification strategy
+///
+/// The strategy to apply `compiler_simp_step` to each node in the circuit from the bottom up (post-order).
+/// Every time a node is simplified, we iterate over the children to make sure that any children we haven't
+/// seen before get recursively fully simplified before continuing.
+/// The final result is a fixed point where no further `compiler_simp_step` simplifications are possible.
 #[pyfunction]
 #[pyo3(name = "compiler_simp")]
 pub fn compiler_simp_py(circ: CircuitRc) -> CircuitRc {
-    compiler_simp(&circ)
+    let mut context = Default::default();
+    compiler_simp(&circ, &mut context)
 }
 
-/// this simplifies individual nodes from the bottom up.
-/// whenever a node is simplified, its grandchildren are simplified, but great grandchildren aren't
-/// this is meant to catch all simplifications given that individual rewrites don't change great grandchildren
-pub fn compiler_simp(circ: &Circuit) -> CircuitRc {
-    cached_circuit_lambda! {
-        let mut recurse = |circ: CircuitRc| -> CircuitRc = {
-            let mut result: CircuitRc = circ.map_children_unwrap(&mut |x:&Circuit|recurse(x.clone().rc())).rc();
-            for iter_count in 0.. {
-                match compiler_simp_step(&result) {
-                    Some(r) => result = compiler_simp_down(&r).unwrap_or(r),
-                    None => {
-                        break;
-                    }
-                }
-                // result.compiler_print();
-                if iter_count > 50 {
-                    println!("here");
-                    result.compiler_print();
-                    compiler_simp_step(&result).unwrap().compiler_print();
-                    panic!();
-                }
+pub fn compiler_simp(circ: &Circuit, opt_context: &mut OptimizationContext) -> CircuitRc {
+    /// check if any new children have not been simplified yet, and simplify them if so
+    fn simplify_changed_descendants(
+        circ: &Circuit,
+        simplified: &mut HashMap<HashBytes, CircuitRc>,
+    ) -> Option<CircuitRc> {
+        circ.map_children_op(&mut |x: &Circuit| {
+            if simplified.contains_key(&x.info().hash) {
+                None
+            } else {
+                Some(fully_simplify(x.clone().rc(), simplified))
             }
-            result
-        };
-    };
-    recurse(circ.clone().rc())
+        })
+        .map(|c| c.rc())
+    }
+    /// fully simplify a circuit and all its descendants recursively until we hit a fixed point
+    fn fully_simplify(
+        circ: CircuitRc,
+        simplified: &mut HashMap<HashBytes, CircuitRc>,
+    ) -> CircuitRc {
+        let mut result: CircuitRc = circ
+            .map_children_unwrap(&mut |x: &Circuit| fully_simplify(x.clone().rc(), simplified))
+            .rc();
+        for iter_count in 0.. {
+            match compiler_simp_step(&result) {
+                Some(r) => result = simplify_changed_descendants(&r, simplified).unwrap_or(r),
+                None => break,
+            }
+            if iter_count > 50 {
+                result.compiler_print();
+                compiler_simp_step(&result).unwrap().compiler_print();
+                panic!();
+            }
+        }
+        simplified.insert(circ.info().hash, result.clone());
+        result
+    }
+    fully_simplify(circ.clone().rc(), &mut opt_context.cache.simplified)
 }
 
 #[pyfunction]
 pub fn compiler_simp_until_same(circ: CircuitRc) -> CircuitRc {
-    apply_fn_until_same(&circ, |x: &CircuitRc| compiler_simp(x))
+    let mut context = Default::default();
+
+    apply_fn_until_same(&circ, |x: &CircuitRc| compiler_simp(x, &mut context))
 }
 
 #[pyfunction]
@@ -355,8 +335,9 @@ pub fn deep_pull_concat_messy(circuit: CircuitRc, min_size: Option<usize>) -> Ci
 
 #[pyfunction]
 pub fn deep_pull_concat(circuit: CircuitRc, min_size: Option<usize>) -> CircuitRc {
+    let mut cache = Default::default();
     let pulled = deep_pull_concat_messy(circuit, min_size);
     apply_fn_until_same(&pulled, |x: &CircuitRc| {
-        deep_push_down_index(compiler_simp(x), min_size)
+        deep_push_down_index(compiler_simp(x, &mut cache), min_size)
     })
 }
