@@ -1,15 +1,15 @@
 #![allow(clippy::borrow_deref_ref)]
+use crate::pyo3_prelude::*;
 use num_bigint::BigUint;
+use pyo3::{exceptions, pyclass::CompareOp};
+use std::hash::Hash;
 use std::{
-    collections::HashSet,
+    collections::{HashMap, HashSet},
     iter::zip,
     ops::{Deref, DerefMut},
     rc::Rc,
 };
 
-use crate::pyo3_prelude::*;
-use pyo3::{exceptions, pyclass::CompareOp};
-
 use std::fmt::Debug;
 use thiserror::Error;
 
@@ -24,6 +24,7 @@ pub mod compiler_strip;
 mod computational_node;
 pub mod concat_rewrite;
 mod constant;
+pub mod debugging;
 pub mod deep_rewrite;
 pub mod diag_rewrite;
 pub mod generalfunction;
@@ -255,6 +256,10 @@ pub trait CircuitNode: CircuitNodePrivate {
         tensors: &[Tensor],
         device_dtype: &TorchDeviceDtype,
     ) -> Result<Tensor, TensorEvalError>;
+
+    fn get_hash(&self) -> HashBytes {
+        self.info().hash
+    }
 }
 
 pub trait CircuitNodeAutoName: CircuitNode {
@@ -685,6 +690,11 @@ impl CachedCircuitInfo {
     pub fn rank(&self) -> usize {
         self.shape.len()
     }
+    pub fn hash_usize(&self) -> usize {
+        let mut hash_prefix: [u8; 8] = Default::default();
+        hash_prefix.copy_from_slice(&self.hash[..8]);
+        usize::from_le_bytes(hash_prefix)
+    }
 }
 
 #[derive(Error, Debug, Clone)]
@@ -1000,6 +1010,35 @@ where
     recurse(circuit)
 }
 
+pub fn apply_fn_cache<I, K, O, F, FK>(i: &I, f: F, c: &mut HashMap<K, O>, fk: FK) -> O
+where
+    F: Fn(&I) -> O,
+    FK: Fn(&I) -> K,
+    O: Clone,
+    K: Eq + Hash,
+{
+    let k = fk(i);
+    match c.get(&k) {
+        Some(r) => r.clone(),
+        None => {
+            let r = f(i);
+            c.insert(k, r.clone());
+            r
+        }
+    }
+}
+
+pub fn deep_map_op_context<F, C>(circuit: &Circuit, f: &F, context: &mut C) -> Option<CircuitRc>
+where
+    F: Fn(&Circuit, &mut C) -> Option<CircuitRc>,
+{
+    let inner_mapped = circuit.map_children_op(|x| deep_map_op_context(x, f, context));
+    match inner_mapped {
+        Some(z) => f(&z, context).or(Some(z.rc())),
+        None => f(&circuit, context),
+    }
+}
+
 pub fn evaluate_fn_uncached(
     circ: &Circuit,
     device_dtype: &TorchDeviceDtype,
