@@ -244,78 +244,6 @@ pub fn rust_expression_notation_circuit(circuit: CircuitRc) -> String {
     source
 }
 
-#[rustfmt::skip]
-#[test]
-fn slightly_test_expression_notation_circuit() {
-    use crate::{
-        circuit::{
-            circuit_utils::replace_all_randn_seeded, Add, ArrayConstant, Concat, Einsum,
-            GeneralFunction, Index, Rearrange, ScalarConstant, Scatter, Symbol,
-        },
-        rearrange_spec::OpSize,
-        rearrange_spec::RearrangeSpec,
-        tensor_util::Slice,
-        tensor_util::{TensorAxisIndex, TensorIndex},
-    };
-    use uuid::uuid;
-    pyo3::prepare_freethreaded_python();
-    let array = ArrayConstant::randn_named_seeded(vec![2, 4, 3, 4], None, 1861206500329984137).rc();
-    let circuit2 = Einsum::nrc(
-        vec![(
-            Add::nrc(
-                vec![
-                    Rearrange::nrc(
-                        Index::nrc(
-                            array.clone(),
-                            TensorIndex(vec![
-                                TensorAxisIndex::Single(0),
-                                TensorAxisIndex::new_plain_slice(0, 2),
-                                TensorAxisIndex::IDENT,
-                                TensorAxisIndex::new_tensor_randint_seeded(4, 4, 0),
-                            ]),
-                            None,
-                        ),
-                        RearrangeSpec::ident(3),
-                        None,
-                    ),
-                    Concat::nrc(vec![ScalarConstant::nrc(2.2, vec![2, 3, 4], None)], 0, None),
-                    Scatter::nrc(
-                        Symbol::nrc(
-                            vec![2, 3, 4],
-                            uuid!("20fd28de-09d9-4773-8f0f-97dcceb855c8"),
-                            None,
-                        ),
-                        TensorIndex::ident(3),
-                        vec![2, 3, 4],
-                        None,
-                    ),
-                ],
-                None,
-            ),
-            vec![0, 1, 2],
-        )],
-        vec![0, 1],
-        None,
-    );
-    println!(
-        "{:?}",
-        array.as_array_constant().unwrap().value.hash_usize()
-    );
-    println!("{}", rust_expression_notation_circuit(circuit2.clone()));
-    let circuit2back = {
-    let node0 = ScalarConstant::nrc(2.2_f64, vec![2,3,4], None);
-    let node1 = Symbol::nrc(vec![2,3,4], uuid!("20fd28de-09d9-4773-8f0f-97dcceb855c8"), None);
-    let node2 = Scatter::nrc(node1.clone(), TensorIndex ( vec![TensorAxisIndex::Slice(Slice { start:None, stop:None }), TensorAxisIndex::Slice(Slice { start:None, stop:None }), TensorAxisIndex::Slice(Slice { start:None, stop:None })] ), vec![2,3,4], None);
-    let node3 = ArrayConstant::randn_named_seeded(vec![2,4,3,4], None, 10357450781205879365).rc();
-    let node4 = Index::nrc(node3.clone(), TensorIndex ( vec![TensorAxisIndex::Single(0), TensorAxisIndex::Slice(Slice { start:Some(0), stop:Some(2) }), TensorAxisIndex::Slice(Slice { start:None, stop:None }), TensorAxisIndex::new_tensor_randint_seeded(4,4,8895353701350677791)] ), None);
-    let node5 = Concat::nrc(vec![node0.clone()], 0, None);
-    let node6 = Rearrange::nrc(node4.clone(), RearrangeSpec::new(vec![vec![0],vec![1],vec![2]], vec![vec![0],vec![1],vec![2]], vec![OpSize::from(None),OpSize::from(None),OpSize::from(None)]), None);
-    let node7 = Add::nrc(vec![node6.clone(), node5.clone(), node2.clone()], None);
-    Einsum::nrc(vec![(node7.clone(), vec![0,1,2])], vec![0,1], None)
-};
-    assert_eq!(replace_all_randn_seeded(circuit2), circuit2back);
-}
-
 pub fn print_circuit_stats(circuit: &Circuit) {
     let mut result = String::new();
     result.push_str(
