@@ -25,15 +25,12 @@ impl RustRepr for i64 {
 }
 impl RustRepr for f64 {
     fn repr(&self) -> String {
-        format!("{}_f64", self.to_string())
+        format!("{}_f64", self)
     }
 }
 impl RustRepr for OpSize {
     fn repr(&self) -> String {
-        format!(
-            "OpSize::from({})",
-            Option::<usize>::from(self.clone()).repr()
-        )
+        format!("OpSize::from({})", Option::<usize>::from(*self).repr())
     }
 }
 impl RustRepr for String {
@@ -43,7 +40,7 @@ impl RustRepr for String {
 }
 impl RustRepr for Uuid {
     fn repr(&self) -> String {
-        format!("uuid!(\"{}\")", self.to_string())
+        format!("uuid!(\"{}\")", self)
     }
 }
 
