@@ -61,7 +61,7 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     use circuit::batching::{batch_inputs_axis_len, batch_largest};
     use circuit::canonicalize::{canonicalize_node_py, deep_canonicalize};
     use circuit::circuit_manipulation::{
-        deep_map_preorder_py, deep_map_py, filter_nodes_py, path_get, replace_nodes,
+        deep_map_preorder_py, deep_map_py, filter_nodes_py, path_get, replace_nodes_py,
         update_nodes_py, update_path_py,
     };
     use circuit::circuit_optimizer::{
@@ -157,7 +157,10 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
 
     m.add_function(wrap_pyfunction!(strip_names, m)?)?;
     m.add_function(wrap_pyfunction!(deep_maybe_distribute_py, m)?)?;
-    m.add_function(wrap_pyfunction!(maybe_distribute, m)?)?;
+    m.add_function(wrap_pyfunction!(
+        circuit::compiler_heuristics::maybe_distribute_py,
+        m
+    )?)?;
     m.add_function(wrap_pyfunction!(einsum_nest_path, m)?)?;
     m.add_function(wrap_pyfunction!(einsum_nest_optimize, m)?)?;
     m.add_function(wrap_pyfunction!(deep_optimize_einsums, m)?)?;
@@ -173,7 +176,10 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     m.add_function(wrap_pyfunction!(total_flops, m)?)?;
     m.add_function(wrap_pyfunction!(total_arrayconstant_size, m)?)?;
 
-    m.add_function(wrap_pyfunction!(optimize_circuit, m)?)?;
+    m.add_function(wrap_pyfunction!(
+        circuit::circuit_optimizer::optimize_circuit_py,
+        m
+    )?)?;
     m.add_function(wrap_pyfunction!(optimize_einsum_spec_cached, m)?)?;
     m.add_function(wrap_pyfunction!(scatter_to_concat, m)?)?;
     m.add_function(wrap_pyfunction!(scheduled_evaluate, m)?)?;
@@ -199,6 +205,10 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     m.add_function(wrap_pyfunction!(add_pull_diags, m)?)?;
     m.add_function(wrap_pyfunction!(einsum_push_down_trace, m)?)?;
     m.add_function(wrap_pyfunction!(concat_repeat_to_rearrange, m)?)?;
+    m.add_function(wrap_pyfunction!(
+        circuit::debugging::opt_eval_each_subcircuit_until_fail,
+        m
+    )?)?;
     m.add_function(wrap_pyfunction!(
         circuit::algebraic_rewrite::add_outer_product_broadcasts_on_top,
         m
@@ -216,7 +226,7 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     m.add_function(wrap_pyfunction!(deep_map_preorder_py, m)?)?;
     m.add_function(wrap_pyfunction!(deep_map_py, m)?)?;
     m.add_function(wrap_pyfunction!(filter_nodes_py, m)?)?;
-    m.add_function(wrap_pyfunction!(replace_nodes, m)?)?;
+    m.add_function(wrap_pyfunction!(replace_nodes_py, m)?)?;
     m.add_function(wrap_pyfunction!(update_nodes_py, m)?)?;
     m.add_function(wrap_pyfunction!(path_get, m)?)?;
     m.add_function(wrap_pyfunction!(update_path_py, m)?)?;
