@@ -262,8 +262,8 @@ impl From<USlice> for TensorAxisIndex {
 /// for now, doesn't support tensors with negatives
 #[derive(Debug, Clone, FromPyObject)]
 pub enum TensorAxisIndex {
+    Tensor(Tensor), // tensor needs to come first so len 1 tensors go to tensor not single
     Single(i64),
-    Tensor(Tensor),
     Slice(Slice),
 }
 
